#!-PERL-
# fm2mif.pl
# Copyright : Norwegian Telecom Research, 1993
# Programming : Jon Stephenson von Tetzchner
# This file can be changed and modified as long as this
# header is kept intact. This code can not be included
# in a commercial product.

# This script is part of the fm2html package.


$| = 1;

$MIF_MAGIC = "<MIFFile";
$MIF_BOOK_MAGIC = "<Book ";
$FM_MAGIC = "<MakerFile";
$FM_BOOK_MAGIC = "<Bookfile";

$EXTENSION      = "htags";

$INFILE = 0;

# *** Get command line parameters
while (@ARGV)
{
    $param = shift;
    if ($param =~ /-title/i || $param =~ /-t/i)
	{
	    $TITLE = shift;
	    print "Title: $TITLE\n";
	}
    elsif ($param =~ /-author/i || $param =~ /-a/i)
	{
	    $AUTHOR = shift;
	    print "Author: $AUTHOR\n";
	}
    elsif ($param =~ /-file/i || $param =~ /-f/i)
	{
	    $INFILE = shift;
	    print "File: $INFILE\n";
	}
    elsif ($param =~ /-xdpi/i)
	{
	    $ENV{'F2HXDPI'} = shift;
	}
    elsif ($param =~ /-ydpi/i)
	{
	    $ENV{'F2HYDPI'} = shift;
	}
    elsif ($param =~ /-ng/i)
	{
	    $ENV{'NOGRAPHICS'} = 1;
	}
    elsif ($param =~ /-ntoc/i)
	{
	    $NOTOC = 1;
	    $ENV{'NOTOC'} = 1;
	}
    elsif ($param =~ /^-/)
	{
	    print "fm2html: Unknown parameter \"$param\". Skipped.\n";
	}
    else
	{
	    $INFILE = $param;
	    print "File: $INFILE\n";
	}
}

if (!$INFILE)
{
  print "\nUsage: fm2html fm-file [-t title] [-a author] [-ng]\n";
  print "\nUsage: fm2html <FrameMaker file> [-title title] [-author author] [-xdpi x] [ydpi y] [-ng]\n";

  print "\n";
  print "\tThe required parameter is the name of a document or book to translat\
e\n";
  print "\tThe \"-title\" option specifies the title of the document or book\n";
  print "\tThe \"-author\" option specifies the author of the document or book\n";
  print "\tThe \"-xdpi\" option specifies the horizontal resolution of graphics \n";
  print "\tThe \"-ydpi\" option specifies the vertical resolution of graphics\n";
  print "\tThe \"-ng\" option indicates that no graphics be converted\n";
  print "\n";
  die;
}


-e "$INFILE" ||
    die "fm2html: File \"$INFILE\" does not exist\n... exiting";

open(f, "<$INFILE") ||
    die "fm2html: Could not open file \"$INFILE\"... exiting\n";
$idline = <f>;
close(f);

if ($idline =~ /^$FM_MAGIC/i || $idline =~ /^$FM_BOOK_MAGIC/i)
{	
    if ($INFILE =~ /\.doc$/)
	{
	    $base = $INFILE;
	    $base =~ s/\.doc$//;
	    $mif = $base.".mif";
	}
    else
	{
	    $base = $INFILE;
	    $mif = $INFILE.".mif";
	}

    print "Executing \"fm2mif $INFILE\"\n";
    system("fm2mif $INFILE");

    print "\nExecuting \"mif2html $mif $TITLE $AUTHOR\"\n";
    &mif2html($mif, $TITLE, $AUTHOR);
}
elsif ($idline =~ /^$MIF_MAGIC/i || $idline =~ /^$MIF_BOOK_MAGIC/i)
{ 
    print "\nExecuting \"mif2html $INFILE $TITLE $AUTHOR\"\n";
    &mif2html($INFILE, $TITLE, $AUTHOR);
}



# *** mif2html 

sub mif2html
{
    local ($INFILE, $TITLE, $AUTHOR) = @_; 

    -e "$INFILE" ||
	die "fm2html: mif2html: File \"$INFILE\" does not exist\n... exiting";
    open(f, "<$INFILE") ||
	die "fm2html: mif2html: Could not open file \"$INFILE\"... exiting\n";
    $idline = <f>;
    close(f);

    if ($INFILE =~ /\.mif/)
	{
	    $mif = $INFILE;
	    $base = $mif;
	    $base =~ s/\.mif$//;
	}
    else
	{
	    $base = $INFILE;
	    $mif = $INFILE.".mif";
	}

    $tagsfile = "$base.$EXTENSION";
    if (!(-e $tagsfile))
	{ 
	    if ( $ENV{'MIF2HTMLTAGSFILE'} )
		{ $defaulttagsfile = $ENV{'MIF2HTMLTAGSFILE'}; }
	    else
		{ $defaulttagsfile = "-LIBDIR-/htags"; }

	    system( "cp $defaulttagsfile $tagsfile" );
	}

    -e $tagsfile || die("fm2html: mif2html: The tags file \"$tagsfile\" could not be opened... exiting.");

    $ENV{'TAGSFilename'} = $tagsfile;
    print "fm2html: mif2html: Using tags file \"$tagsfile\"\n";

    if ($idline =~ /$MIF_MAGIC/i)
	{ &makerfile ($base, $TITLE, $AUTHOR); }
    elsif ($idline =~ /$MIF_BOOK_MAGIC/i)
	{ &bookfile ($base, $TITLE, $AUTHOR); }
}


# *** Makerfile - Handles a mif file.
sub makerfile
{
    local ($INFILE, $TITLE, $AUTHOR) = @_;

    $LANG = $ENV{'DOCUMENTLANGUAGE'};
    if ($LANG eq '') 
	{ 
	    $LANG = "US"; 
	    $ENV{'DOCUMENTLANGUAGE'} = "US";
	} 

    print "fm2html: starting translation of $INFILE.mif to $INFILE.html\n";
    print "fm2html: Language: $LANG\n";

    open(titlefile, "> $INFILE.tmp.000L") ||
	die "fm2html: Could not open title file \"$INFILE.tmp.000L\"... exiting";
    if ($TITLE ne '')
	{
	    print titlefile "<TITLE>$TITLE</TITLE>";
	    print titlefile "<H1>$TITLE</H1>";
	    print titlefile "<ADDRESS>$AUTHOR</ADDRESS>";
	} 
    close(titlefile);

    if (!$NOTOC)
	{
	    open(indexfile, "> $INFILE.tmp.000I") ||
		die "fm2html: Could not open index file \"$INFILE.tmp.000I\"... exiting";
	    if ($LANG eq "NO")
		{ print indexfile "<HR><H2>Innhold</H2>\n"; }
	    else
		{ print indexfile "<HR><H2>Table of Contents</H2>\n"; }    
	    close(indexfile);
	}

    system("rm -f $INFILE.tmp.000HT");
    system("f2htranslate $INFILE $INFILE.tmp.000HT $INFILE.tmp.000I $INFILE.tmp.000L \"$TITLE\"");

    open(indexfile, ">> $INFILE.tmp.000I") ||
	die "fm2html: Could not reopen index file \"$INFILE.tmp.000I\"... exiting";
    print indexfile "<HR>";
    close(indexfile);

    system("cat $INFILE.tmp.000L $INFILE.tmp.000I | insertcharacters > $INFILE.html");
    system("cat $INFILE.tmp.000HT >> $INFILE.html");
    system("rm -f $INFILE.tmp.000I $INFILE.tmp.000L $INFILE.tmp.000HT");
}


# *** Bookfile - Handles a mif book.
sub bookfile 
{
    local($INFILE, $TITLE, $AUTHOR) = @_;

    $LANG = $ENV{'DOCUMENTLANGUAGE'};
    if ($LANG eq '') 
	{
	    $LANG = "US";
	} 

    $ENV{'BOOKFilename'} = $INFILE;
    $ENV{'BOOKTitle'} = $TITLE;

    print "fm2html: Starting translation of FrameMaker book file $INFILE.mif (and all documents in it) to $INFILE.html\n";
    print "fm2html: Language: $LANG\n";

    system ("cp -LIBDIR-/fm2html-toc.gif .");
    system ("cp -LIBDIR-/fm2html-next.gif .");
    system ("cp -LIBDIR-/fm2html-previous.gif .");

    open(titlefile, "> $INFILE.tmp.000L") ||
	die "fm2html: Could not open title file \"$INFILE.tmp.000L\"... exiting";

    print titlefile "<!-- This file was created with the fm2html filter.\n";
    print titlefile "The filter is copyright Norwegian Telecom Research and\n";
    print titlefile "was programmed by Jon Stephenson von Tetzchner.  -->\n";

    if ($TITLE ne '')
	{
	    print titlefile "<TITLE>$TITLE</TITLE>\n";
	    print titlefile "<H1>$TITLE</H1>\n";
	    print titlefile "<ADDRESS>$AUTHOR</ADDRESS>\n";
	} 
    close(titlefile);

    if (!$NOTOC)
	{
	    open(indexfile, "> $INFILE.tmp.000I") ||
		die "fm2html: Could not open index file \"$INFILE.tmp.000I\"... exiting";
	    if ($LANG eq "NO")
		{ print indexfile "<HR><H2>Innhold</H2>\n"; }
	    else
		{ print indexfile "<HR><H2>Table of Contents</H2>\n"; }    
	    close(indexfile);
	}

    system("book < $INFILE.mif");

    if (!$NOTOC)
	{
	    open(indexfile, ">> $INFILE.tmp.000I") ||
		die "fm2html: Could not reopen index file \"$INFILE.tmp.000I\"... exiting";
	    print indexfile "<HR>";
	    close(indexfile);
	}

    system("cat $INFILE.tmp.000L $INFILE.tmp.000I | insertcharacters >$INFILE.html");
    system("rm $INFILE.tmp.000I $INFILE.tmp.000L ");
}
