/*********************************************************/
/* trans_table-code.c						 */
/* Copyright : Norwegian Telecom Research, 1993		 */
/* Programming : Jon Stephenson von Tetzchner		 */
/* This file can be changed and modified as long as this */
/* header is kept intact. This code can not be included	 */
/* in a commercial product.				 */
/*********************************************************/

#include "trans.h"

int in_table_title = FALSE;
int in_table_header = FALSE;
int in_table_footer = FALSE;
struct FMTable *first_table = NULL;
struct FMTable *current_table = NULL;
struct FMRow *current_row = NULL;
struct FMCell *current_cell = NULL;
struct FMCellLine *current_cell_line = NULL;
int cell_no = 0;


/*******************************************************************************
  void GetTableID() :
 *******************************************************************************/

void GetTableID()
{
    current_table->id = GetID();
}


/*******************************************************************************
  void GetTableNumColumns() :
 *******************************************************************************/

void GetTableNumColumns()
{
    current_table->num_columns = GetID();
    current_table->column_widths = (int *) calloc(current_table->num_columns, sizeof(int));
    /* fprintf(stderr, "Table Column number: %d\n", current_table->num_columns); */
}


/*******************************************************************************
  void TableStart() :
 *******************************************************************************/

void TableStart()
{
    current_table = (struct FMTable *) calloc(1, sizeof(struct FMTable));
}


/*******************************************************************************
  void TableEnd() :
 *******************************************************************************/

void TableEnd()
{
    int i;
    for (i = 1; i<= footnote_def_nr; i++)
	{
	    current_table->footnote[i] = footnote[i];
	    /* footnote[i] = 0; */
	}
    current_table->footnote_def_nr = footnote_def_nr;
    footnote_def_nr = 0;
    footnote_use_nr = 0;
    current_table->next_table = first_table;
    first_table = current_table;
    current_table = NULL;
    /* InsertTable(stderr, current_table->id); */
}


/*******************************************************************************
  void TableTitleStart() :
 *******************************************************************************/

void TableTitleStart()
{
    in_table_title = TRUE;
}


/*******************************************************************************
  void TableTitleEnd() :
 *******************************************************************************/

void TableTitleEnd()
{
    in_table_title = FALSE;
}


/*******************************************************************************
  void TableHeadStart() :
 *******************************************************************************/

void TableHeadStart()
{
    in_table_header = TRUE;
}


/*******************************************************************************
  void TableHeadEnd() :
 *******************************************************************************/

void TableHeadEnd()
{
    in_table_header = FALSE;
}


/*******************************************************************************
  void TableFootStart() :
 *******************************************************************************/

void TableFootStart()
{
    in_table_footer = TRUE;
}


/*******************************************************************************
  void TableFootEnd() :
 *******************************************************************************/

void TableFootEnd()
{
    in_table_footer = FALSE;
}


/*******************************************************************************
  void TableRowStart() :
 *******************************************************************************/

void TableRowStart()
{
    if (in_table_header)
	{
	    current_row = current_table->header = (struct FMRow *) calloc(1, sizeof(struct FMRow));
	}
    else if (in_table_footer)
	{
	    current_row = current_table->footer = (struct FMRow *) calloc(1, sizeof(struct FMRow));
	}
    else
	{
	    struct FMRow *new_row = (struct FMRow *) calloc(1, sizeof(struct FMRow));
	    if (!current_table->row)
		current_row = current_table->row = new_row;
	    else
		{
		    current_row->next_row = new_row;
		    current_row = new_row;
		}	
	}
}    


/*******************************************************************************
  void TableRowEnd() :
 *******************************************************************************/

void TableRowEnd()
{
    cell_no = 0;
}


/*******************************************************************************
  void TableCellStart() :
 *******************************************************************************/

void TableCellStart()
{
    struct FMCell *new_cell = (struct FMCell *) calloc(1, sizeof(struct FMCell));
    if (!current_row->cell)
	current_cell = current_row->cell = new_cell;
    else
	{
	    current_cell->next_cell = new_cell;
	    current_cell = new_cell;
	}	
}	


/*******************************************************************************
  void TableCellEnd() :
 *******************************************************************************/

void TableCellEnd()
{
    EndParagraph();
    cell_no++;
}


/*******************************************************************************
  int GetID() :
        Input  : Uses yytext
	Output : Number not quoted.
 *******************************************************************************/

int GetID()
{
    int i = 0;

    while(yytext[i] && yytext[i] != 32)
	{
	    i++;
	}

    if (yytext[i] == 32)
	{
	    i++;
	    return atoi(&(yytext[i]));
	}			
    else return 0;
}


/*******************************************************************************
 void StringLength() : Returns length of string, conting special characters
                       like "\xbf " one. (and html tags like 0...)
 *******************************************************************************/

int StringLength(str)
      char* str;
{
    int i = 0;
    int lth = 0;
    if (str )
	while (str[i]) {
	    if (str[i] == '\\') i+=4;
	    else if (str[i] == '<') {
		int j = i++;
		while (str[i] && str[i] != '>') i++;
		if (!str[i]) i=j;
		else lth--;
	    }
	    i++;
	    lth++;
	}
    return lth;
}


/*******************************************************************************
 void NewCellLine() : 
 *******************************************************************************/

void NewCellLine()
{
    if (in_string)
	{
	    PrintHeaderTypeEnd();
	}
    if (current_cell) {
	struct FMCellLine *new_line = (struct FMCellLine *) calloc(1, sizeof(struct FMCellLine));
	current_cell->num_lines ++;
	if (current_cell->num_lines > current_row->max_num_lines)
	    current_row->max_num_lines = current_cell->num_lines;
	    
	new_line->fmline = NULL;

	if (!current_cell->line) 
	    current_cell->line = new_line;
	else
	    {
		struct FMCellLine *temp1, *temp2;
		temp1 = current_cell->line;
		temp2 = temp1->next_line;
		while(temp2)
		    {
			temp1 = temp2;
			temp2 = temp1->next_line;
		    }
		temp1->next_line = new_line;
	    }
	current_cell_line = new_line;
    }
}


/*******************************************************************************
 void UseString() : 
 *******************************************************************************/

void UseString( string_contents )
      char *string_contents;
{
    int str_lth;

    if (in_footnote)
	{
	    if (footnote_def_nr >= MAXNUMFOOTNOTES)
		{
		    fprintf(stderr, "\nERROR: Too many footnotes.\n\n");
		    return;
		}
	    strcat(footnote[footnote_def_nr], string_contents);
/*	    free(string_contents); */
	}		    
    else if (in_table_title)
	{
	    if (current_table->table_title)
		{
		    char *temp1;
		    temp1 = current_table->table_title;
		    current_table->table_title = calloc(strlen(temp1) + 
							strlen(string_contents) + 1, sizeof(char));
		    strcpy(current_table->table_title, temp1);
		    strcat(current_table->table_title, string_contents);
		    free(temp1);
		  /*  free(string_contents); */
		}
	    else
		{
		    current_table->table_title = calloc(strlen(para_num_string) + 
							strlen(string_contents) + 1, sizeof(char));
		    strcat(current_table->table_title, string_contents);
		}
	}
    else if (current_cell && current_cell_line)
	{
	    if (current_cell_line->fmline)
		{
		    char *temp1;
		    temp1 = current_cell_line->fmline;
		    current_cell_line->fmline = calloc(strlen(temp1) + 
						       strlen(string_contents) + 1, sizeof(char));
		    strcpy(current_cell_line->fmline, temp1);
		    strcat(current_cell_line->fmline, string_contents);
		    free(temp1);
/*		    free(string_contents); */
		}
	    else
		{
		    current_cell_line->fmline = calloc(strlen(string_contents) + 1, sizeof(char));
		    strcpy(current_cell_line->fmline, string_contents);
		}

	    str_lth = StringLength(current_cell_line->fmline);
	    if (str_lth + 2 > current_table->column_widths[cell_no])
		current_table->column_widths[cell_no] = str_lth + 2;
	}
}


/*******************************************************************************
 void PrintTableTitle() : 
 *******************************************************************************/

void PrintTableTitle(f, table, max_lth)
      FILE *f;
      struct FMTable *table;
      int max_lth;
{
    int i, indent;
    char *str = table->table_title;
    char line[200];
    char tmpline[300];
    char *word;

    int IsNotFirstLine = 0;

    tmpline[0] = 0;
    line[0] = 0;

    word = strtok(str, " \t");
    while (word)
	{
	    strcat (tmpline, word);
	    strcat (tmpline, " ");
	    if (StringLength(tmpline) < max_lth)
		{
		    strcat (line, word);
		    strcat (line, " ");
		}
	    else
		{
		    if (strlen(line) > 0)
			{
			    if (IsNotFirstLine) fprintf(f, "\t\t");
			    fprintf(f, "%s\n", line);
			    strcpy(line, word);
			    tmpline[0] = 0;
			    IsNotFirstLine = 1;
			}
		    else
			{
			    if (IsNotFirstLine) fprintf(f, "\t\t");
			    fprintf(f, "%s\n", tmpline);
			    tmpline[0] = 0;
			    IsNotFirstLine = 1;
			}
		}
	    word = strtok(NULL, " \t");
	}
    if (strlen(line) > 0)
	{
	    if (IsNotFirstLine) fprintf(f, "\t\t");
	    fprintf(f, "%s\n", line);
	}
}


/*******************************************************************************
 void InsertTable() : 
 *******************************************************************************/

void InsertTable(f, id)
      FILE *f;
      int id;
{
    struct FMTable *table;
    struct FMRow *row;
    int table_width = 0;
    int i;

    table = FindTable(id);	/* Find the table to print */
    if (!table)
	{
	    fprintf(stderr, "Program error. Could not print table.\n");
	    exit(-1);
	}

    /* fprintf(stderr, "Column widths : ");
    for (i=0; i < table->num_columns; i++)
	fprintf(stderr, "%d x  ", table->column_widths[i]);
    fprintf(stderr, "\n"); */

    fprintf(f, "<pre>\n");

    for (i = 0; i < table->num_columns; i++)
	table_width += table->column_widths[i];

    if (table->table_title)
	PrintTableTitle(f, table, 80);		/* Print the table title */

    for (i = 0; i < table_width - 2; i++)	       	/* Print "-------------" */
	fputc('-', f);
    fputc('\n',f );

    if (table->header) 
	{
	    PrintTableRow(f, table->header, table->column_widths);  /* Print the table header */

	    for (i = 0; i < table_width - 2; i++)		    /* Print "--------------" */
		fputc('-',f );
	    fputc('\n', f);
	}
    row = table->row;
    while (row)					        /* Print the table rows */
	{
	    PrintTableRow(f, row, table->column_widths);
	    row = row->next_row;
	}

    for (i = 0; i < table_width - 2; i++)	  	/* Print "--------------" */
	fputc('-', f);
    fputc('\n', f);

    if (table->footer) 
	{
	    PrintTableRow(f, table->footer, table->column_widths);  /* Print the table footer */

	    for (i = 0; i < table_width - 2; i++)		    /* Print "--------------" */
		fputc('-',f );
	    fputc('\n', f);
	}
    fprintf(f, "</pre>\n");
    if (table->footnote_def_nr) PrintTableFootnotes(f, table);
    table->footnote_def_nr = 0;
    /* FreeTable(table); */
}
	

/*******************************************************************************
 void PrintTableRow() : 
 *******************************************************************************/
    
void PrintTableRow(f, row, column_widths)
      FILE *f;
      struct FMRow *row;
      int *column_widths;
{
    int i, column;
    
    for (i = 0; i < row->max_num_lines; i++)
	{
	    struct FMCell *fmcell = row->cell;
	    column = 0;
	    while (fmcell)
		{
		    PrintTableCell(f, fmcell, i, column_widths[column]);
		    fmcell = fmcell->next_cell;
		    column++;
		}
	    fprintf(f, "\n");
	}
}


/*******************************************************************************
 void PrintTableCell() : 
 *******************************************************************************/
    
void PrintTableCell(f, cell, line_nr, width)
      FILE *f;
      struct FMCell *cell;
      int line_nr, width;
{
    struct FMCellLine *line = cell->line;
    char outstring[100];
    int i = 0;
    while (line && i++ < line_nr) line = line->next_line;

    if (line && line->fmline)
	{
	    int cell_width;
	    cell_width = width + strlen(line->fmline) - StringLength(line->fmline);
	    sprintf(outstring, "%%-%ds", cell_width);
	    fprintf(f, outstring, line->fmline);
	}
    else
	{
	    char *empty = " ";
	    sprintf(outstring, "%%-%ds", width);
	    fprintf(f, outstring, empty);
	    if (line && !line->fmline)
		/*printf("Ooops ... empty fmline (?) ...\n") */  ;
	}
}


/*******************************************************************************
 FMTable *FindTable() : 
 *******************************************************************************/

struct FMTable *FindTable(id)
      int id;
{
    struct FMTable *stable;

    stable = first_table;

    while (stable && stable->id != id)
	stable = stable->next_table;
    if (stable) return stable;
    return NULL;
}


/*******************************************************************************
 void PrintTableFootnotes() : 
 *******************************************************************************/

void PrintTableFootnotes(f, table)
      FILE *f;
      struct FMTable *table;
{
    int i;
    fprintf(f, "<DL COMPACT>\n");
    for (i=1; i<=table->footnote_def_nr; i++)
	{
	    fprintf(f, "<DT><A NAME=FN%d%c>(%c)</A><DD>%s\n", table->id,  
		    i + 'a' - 1, i + 'a' - 1, table->footnote[i]);
	    free(table->footnote[i]);
	}
    fprintf(f, "</DL>\n");
}









